@@ -9,7 +9,7 @@ import abc
 import inspect
 import multiprocessing as mp
 from abc import ABC
-from typing import List, Union, Collection
+from typing import Collection, List, Union
 
 from . import connectors, exceptions
 
@@ -26,11 +26,7 @@ def _get_nodes_from_connectors(connector_list: Collection[connectors.AbstractCon
 
     nodes = []
     for c in connector_list:
-        if isinstance(c, connectors.Output) and c.is_connected:
-            nodes.append(c.get_partner().parent_node)
-
-        if isinstance(c, connectors.Input) and c.is_connected:
-            nodes.extend(p.parent_node for p in c.get_partners())
+        nodes.extend(p.parent_node for p in c.get_partners())
 
     return nodes
 
@@ -50,9 +46,12 @@ class AbstractNode(abc.ABC):
         self._states = mp.Manager().dict({id(p): False for p in self._processes})
 
         self._current_process_state = False
-        for connection in self._get_attrs(connectors.AbstractConnector):
+        for connection in self.get_connectors():
             connection._node = self
 
+    def get_connectors(self) -> List[connectors.AbstractConnector]:
+        return self._get_attrs(connectors.AbstractConnector)
+
     @property
     def num_processes(self) -> int:
         """The number of processes assigned to the current node"""
@@ -107,7 +106,7 @@ class AbstractNode(abc.ABC):
             MissingConnectionError: For an invalid instance construction
         """
 
-        for conn in self._get_attrs(connectors.AbstractConnector):
+        for conn in self.get_connectors():
             if not conn.is_connected:
                 raise exceptions.MissingConnectionError(
                     f'Connector {conn} does not have an established connection (Node: {conn.parent_node})')
@@ -159,15 +158,29 @@ class AbstractNode(abc.ABC):
         """Return whether the node is still expecting data from upstream"""
 
         for input_connector in self._get_attrs(connectors.Input):
-            if not input_connector.empty():
-                return True
-
+            # IMPORTANT: The order of the following code blocks is crucial
+            # We check for any running upstream nodes first
             for partner in input_connector.get_partners():
                 if not partner.parent_node.node_finished:
                     return True
 
+            # Check for any unprocessed data once we know there are no
+            # nodes still populating any input queues
+            if not input_connector.empty():
+                return True
+
         return False
 
+    def __str__(self) -> str:  # pragma: no cover
+        return f'<{self.__repr__()} object at {hex(id(self))}>'
+
+    def __repr__(self) -> str:  # pragma: no cover
+        return f'{self.__class__.__name__}(num_processes={self.num_processes})'
+
+    def __del__(self):
+        if any(p.is_alive() for p in self._processes):
+            raise RuntimeError(f'Cannot delete a node while it is running (del called on node {self})')
+
 
 class Source(AbstractNode, ABC):
     """A pipeline process that only has output streams"""
@@ -219,7 +232,7 @@ class Node(Target, Source, ABC):
             OrphanedNodeError: For an instance that is inaccessible by connectors
         """
 
-        if not self._get_attrs(connectors.AbstractConnector):
+        if not self.get_connectors():
             raise exceptions.OrphanedNodeError('Node has no associated connectors and is inaccessible by the pipeline.')
 
         self._validate_connections()
